@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2023 Martin Donath <martin.donath@squidfunk.com>
+# Copyright (c) 2016-2024 Martin Donath <martin.donath@squidfunk.com>
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to
@@ -47,6 +47,7 @@ class SearchPlugin(BasePlugin[SearchConfig]):
         super().__init__(*args, **kwargs)
 
         # Initialize incremental builds
+        self.is_dirty = False
         self.is_dirtyreload = False
 
         # Initialize search index cache
@@ -58,6 +59,10 @@ class SearchPlugin(BasePlugin[SearchConfig]):
 
     # Initialize plugin
     def on_config(self, config):
+        if not self.config.enabled:
+            return
+
+        # Retrieve default value for language
         if not self.config.lang:
             self.config.lang = [self._translate(
                 config, "search.config.lang"
@@ -70,7 +75,7 @@ class SearchPlugin(BasePlugin[SearchConfig]):
             )
 
         # Retrieve default value for pipeline
-        if not self.config.pipeline:
+        if self.config.pipeline is None:
             self.config.pipeline = list(filter(len, re.split(
                 r"\s*,\s*", self._translate(config, "search.config.pipeline")
             )))
@@ -104,6 +109,10 @@ class SearchPlugin(BasePlugin[SearchConfig]):
 
     # Add page to search index
     def on_page_context(self, context, *, page, config, nav):
+        if not self.config.enabled:
+            return
+
+        # Index page
         self.search_index.add_entry_from_context(page)
         page.content = re.sub(
             r"\s?data-search-\w+=\"[^\"]+\"",
@@ -113,6 +122,10 @@ class SearchPlugin(BasePlugin[SearchConfig]):
 
     # Generate search index
     def on_post_build(self, *, config):
+        if not self.config.enabled:
+            return
+
+        # Write search index
         base = os.path.join(config.site_dir, "search")
         path = os.path.join(base, "search_index.json")
 
@@ -151,7 +164,7 @@ class SearchIndex:
 
     # Add page to search index
     def add_entry_from_context(self, page):
-        search = page.meta.get("search", {})
+        search = page.meta.get("search") or {}
         if search.get("exclude"):
             return
 
@@ -202,10 +215,10 @@ class SearchIndex:
             entry["tags"] = []
             for name in tags:
                 if name and isinstance(name, (str, int, float, bool)):
-                    entry["tags"].append(name)
+                    entry["tags"].append(str(name))
 
         # Set document boost
-        search = page.meta.get("search", {})
+        search = page.meta.get("search") or {}
         if "boost" in search:
             entry["boost"] = search["boost"]
 
@@ -299,9 +312,9 @@ class Element:
     """
 
     # Initialize HTML element
-    def __init__(self, tag, attrs = {}):
+    def __init__(self, tag, attrs = None):
         self.tag   = tag
-        self.attrs = attrs
+        self.attrs = attrs or {}
 
     # String representation
     def __repr__(self):
@@ -437,16 +450,15 @@ class Parser(HTMLParser):
                 return
 
         # Render opening tag if kept
-        if not self.skip.intersection(self.context):
-            if tag in self.keep:
+        if not self.skip.intersection(self.context) and tag in self.keep:
 
-                # Check whether we're inside the section title
-                data = self.section.text
-                if self.section.el in self.context:
-                    data = self.section.title
+            # Check whether we're inside the section title
+            data = self.section.text
+            if self.section.el in self.context:
+                data = self.section.title
 
-                # Append to section title or text
-                data.append(f"<{tag}>")
+            # Append to section title or text
+            data.append(f"<{tag}>")
 
     # Called at the end of every HTML tag
     def handle_endtag(self, tag):
@@ -475,29 +487,28 @@ class Parser(HTMLParser):
             return
 
         # Render closing tag if kept
-        if not self.skip.intersection(self.context):
-            if tag in self.keep:
+        if not self.skip.intersection(self.context) and tag in self.keep:
 
-                # Check whether we're inside the section title
-                data = self.section.text
-                if self.section.el in self.context:
-                    data = self.section.title
+            # Check whether we're inside the section title
+            data = self.section.text
+            if self.section.el in self.context:
+                data = self.section.title
 
-                # Search for corresponding opening tag
-                index = data.index(f"<{tag}>")
-                for i in range(index + 1, len(data)):
-                    if not data[i].isspace():
-                        index = len(data)
-                        break
+            # Search for corresponding opening tag
+            index = data.index(f"<{tag}>")
+            for i in range(index + 1, len(data)):
+                if not data[i].isspace():
+                    index = len(data)
+                    break
 
-                # Remove element if empty (or only whitespace)
-                if len(data) > index:
-                    while len(data) > index:
-                        data.pop()
+            # Remove element if empty (or only whitespace)
+            if len(data) > index:
+                while len(data) > index:
+                    data.pop()
 
-                # Append to section title or text
-                else:
-                    data.append(f"</{tag}>")
+            # Append to section title or text
+            else:
+                data.append(f"</{tag}>")
 
     # Called for the text contents of each tag
     def handle_data(self, data):
@@ -533,6 +544,8 @@ class Parser(HTMLParser):
         elif data.isspace():
             if not self.section.text or not self.section.text[-1].isspace():
                 self.section.text.append(data)
+            elif "pre" in self.context:
+                self.section.text.append(data)
 
         # Handle everything else
         else:
